from collections.abc import MutableMapping

class Bijection(MutableMapping):
    """
        A tout moment, self[x] == self[y] => x == y
    """
    def inv(self, x):
        """
            Retourne y tel que self[y] == x
        """
        ...

# Version par héritage
class Bijection(dict):
    def __setitem__(self, key, value):
        for k, v in self.items():
            if v == value and key != k:
                raise ValueError("Not bijective operation")
        super().__setitem__(key, value)

    def inv(self, x):
        for k,v in self.items():
            if v == x:
                return k


# Version par composition

class Bijection(Bijection):
    def __init__(self):
        self.direct = dict()
        self.reverse = dict()

    def __delitem__(self, key):
        v = self.direct[key]
        del self.direct[key]
        del self.reverse[v]

    def __getitem__(self, key):
        return self.direct[key]

    def __iter__(self):
        return iter(self.direct)

    def __len__(self):
        return len(self.direct)

    def __setitem__(self, key, value):
        if value in self.reverse and key != self.reverse[value]:
            raise ValueError("Not bijective operation")
        self.direct[key] = value
        self.reverse[value] = key

    def inv(self, x):
        return self.reverse[x]
